<?php
/**
 * Core Design Ajax Admin Utility (AJAU) plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	System
 * @category	Plugin
 * @version		1.0.5
 * @copyright	Copyright (C) 2007 - 2009 Core Design, http://www.greatjoomla.com
 * @license		http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.file');
// end

/**
 * Joomla! Core Design AJAU plugin
 *
 * @author		Daniel Rataj <info@greatjoomla.com>
 * @package		Core Design
 * @subpackage	System
 */
class plgSystemCdAjau extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @access	protected
	 * @param	object		$subject The object to observe
	 * @since	1.0
	 */
	function plgSystemCdAjau(&$subject)
	{
		parent::__construct($subject);

		// load plugin parameters
		$this->plugin = &JPluginHelper::getPlugin('system', 'cdajau');
		$this->params = new JParameter($this->plugin->params);
	}
	
	// Call Joomla! function
	function onAfterInitialise() {
		global $mainframe;
		
		$ajau_ping	 = $mainframe->getUserStateFromRequest('ajau', 'ajau', 0);

		if (!$ajau_ping) {
			$mainframe->setUserState('ajau', JRequest::getCmd('ajau', '', 'post'));
		}
	}
	
	/**
	 * Call plugin
	 */
	function onAfterDispatch()
	{
		global $mainframe;
		
		// Disable plugin if user is non-logged or there is administration interface
		if (!plgSystemCdAjau::isLogged() or !$mainframe->isAdmin()) {
			return;
		}

		$load_ajau_extension = plgSystemCdAjau::loadAjauExtension();
		
		// if there is no extension to load, disable plugin
		if (!$load_ajau_extension) {
			return;
		}
		
		// if IE is 6, disable plugin
		if ($this->checkIE6()) {
			return;
		}

		// load language
		JPlugin::loadLanguage('plg_system_cdajau', JPATH_ADMINISTRATOR);
		
		// check Scriptegrator
		if (!defined('_JSCRIPTEGRATOR')) {
			JError::raiseNotice('', JText::_('AJAU_ENABLE_SCRIPTEGRATOR'));
			return;
		}

		// requires Scriptegrator version 1.3.3 or higher
		$version = '1.3.3';
		if (!JScriptegrator::versionRequire($version)) {
			JError::raiseNotice('', JText::sprintf('AJAU_SCRIPTEGRATOR_REQUIREVERSION', $version));
			return;
		}

		// check jQuery
		if (!JScriptegrator::checkLibrary('jquery', 'admin'))
		{
			JError::raiseNotice('', JText::_('AJAU_MISSING_JQUERY'));
			return;
		}
				
		// get option - com_content, ..., methot "GET"
		$get_option = JRequest::getCmd('option', '', 'get');

		// specify path to the "cdajau" folder
		$ajau_folder = dirname(__FILE__) . DS . 'cdajau';

		// specify extension file path
		$ext_folder_path = $ajau_folder . DS . 'extensions' . DS . $get_option;
		
		$class_file = $ext_folder_path . DS . 'class.php';
		
		// load class file
		if (JFile::exists($class_file)) {
			
			$classname = 'AJAU';
			
			JLoader::register($classname, $class_file);
			
			// disable plugin
			if (is_callable(array($classname, 'disable'))) {
				
				$disable = false; // set $disable variable to false - boolean
				
				$disable = call_user_func(array($classname, 'disable'));
				if ($disable) {
					return;
				}
			}
			
			// language strings
			if (is_callable(array($classname, 'language'))) {
				$language_strings = array(); // set language variable to array
				
				// load languages array from class
				$language_strings = call_user_func(array($classname, 'language'));
				
				// set language strings to header
				if ($language_strings) {
					// load extension language
					$this->extLanguage($get_option, $ext_folder_path);
					$this->languageStrings($language_strings); // insert language strings to header
				}
				
			}
			
			// option strings
			if (is_callable(array($classname, 'option'))) {
				$option_strings = array(); // set language variable to array
				
				// load option array from class
				$option_strings = call_user_func(array($classname, 'option'));
				
				// set language strings to header
				if ($option_strings) {
					$this->getOption($option_strings); // insert options strings to header
				}
			}
			
			// load the necessary helpers and scripts
			$this->addPluginHelper($ext_folder_path);
			
			$this->loadAppFramework($ajau_folder);
	
			// load the necessary helpers and scripts
			$this->addPluginScrips($get_option, $ext_folder_path);
	
			if ($load_ajau_extension) {
				$this->addAjauInfo($get_option);
			}
			
			// AJAU tasks
			$ajau_task = $mainframe->getUserStateFromRequest('ajau', 'ajau', 0);
			if ($ajau_task) {
				if ($ajau_task == 'ping') {
					$this->clearQueue();
					$this->close($this->ping('form'));
				} else {
					$msg = null;
					if (is_callable(array($classname, 'runTask'))) {
						$msg = call_user_func(array($classname, 'runTask'), $ajau_task);
					}
					$this->clearQueue();
					$this->close($msg);
				}
			}
		}
		
	}
	
	/*
	 * Close wrapper
	 * 
	 * Close wrapper to the $mainframe->close() function
	 * 
	 * return void
	 */
	function close($msg = ''){
	    global $mainframe;
	    echo $msg;
	    $mainframe->close();
	}
	
	
	/*
	 * Check if user is logged or not
	 */
	function isLogged() {
        $user = &JFactory::getUser();
        return (!$user->get('guest')) ? true : false;
    }
	
    /*
	 * Specify is AJAU is loaded or doesn't
	 */
	function loadAjauExtension() {
		// get option - com_content, ..., methot "GET"
		$get_option = JRequest::getCmd('option', 'index', 'get');
		
		// specify path to the "cdajau" folder
		$ajau_folder = dirname(__FILE__) . DS . 'cdajau';

		// specify extension file path
		$ext_folder_path = $ajau_folder . DS . 'extensions' . DS . $get_option;
		
		if (JFolder::exists($ext_folder_path)) {
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Added info message if AJAU is enabled
	 */
	function addAjauInfo($get_option) {
		$ajax_notice = "<script type=\"text/javascript\">
            <!--
			jQuery(document).ready(function($){
				$('span.version').append(' " . JText::_('AJAU'). "');
				
				// initiate plugin application
				$.ajau_" . $get_option . ".initiator();
			});
			// -->
			</script>";
		$document = &JFactory::getDocument(); // set document for next usage
		$document->addCustomTag($ajax_notice);
	}

	/*
	 * Insert language strings to header
	 */
	function languageStrings($languages = array()) {
		if (!is_array($languages) or empty($languages)) {
			return;
		}
		
		$lang = '';

		$lang .= "<script type=\"text/javascript\">\n";
		$lang .= "<!--\n";
		$lang .= "// AJAU language strings\n";

		$lang .= "jQuery(document).ready(function($) {";
		$lang .="\n";
		$lang .= "	$.extend( $.ajau, { language: {";
		$lang .="\n";

		foreach ($languages as $language) {
			
			if (JText::_($language) == $language) {
				$language = '';
			}
			if ($language) {
				$lang .= '		' . $language . ' : "' . JText::_($language) . '",';
				$lang .="\n";

			}
		}
		$lang = preg_replace('#,\n$#', '', $lang); // remove "," from end
		$lang .="\n";
		$lang .= "	}});";
		$lang .="\n";
		$lang .="});";
		$lang .="\n";
		$lang .= "// -->\n";
		$lang .= "</script>\n";

		$document = &JFactory::getDocument(); // set document for next usage
		$document->addCustomTag($lang);
	}

	/*
	 * Insert option variables to header
	 */
	function getOption($option_array = array()) {
		if (!is_array($option_array) or empty($option_array)) {
			return;
		}

		$options = '';

		$options .= "<script type=\"text/javascript\">\n";
		$options .= "<!--\n";
		$options .= "// AJAU options\n";

		$options .= "jQuery(document).ready(function($) {";
		$options .="\n";
		$options .= "	$.extend( $.ajau, { settings: {";
		$options .="\n";

		foreach ($option_array as $option) {
			$string = explode('|', $option);
			$option_name = ( isset($string[0]) ? $string[0] : '' );
			$value = ( isset($string[1]) ? $string[1] : '' );

			$options .= '		' . $option_name . ' : "' . $value . '",';
			$options .="\n";
		}

		$options = preg_replace('#,\n$#', '', $options); // remove "," from end
		$options .="\n";
		$options .= "	}});";
		$options .="\n";
		$options .="});";
		$options .="\n";
		$options .= "// -->\n";
		$options .= "</script>\n";

		$document = &JFactory::getDocument(); // set document for next usage
		$document->addCustomTag($options);
	}

	/*
	 * Add neccessary scripts and styles
	 */
	function loadAppFramework($ajau_folder) {
		$live_path = JURI::root(true) . '/'; // define live site
		
		// attach JS
		$scripts = array();
		$scripts [] .= 'jquery.ajau.js';
		$scripts [] .= 'facebox/facebox.js';

		$document = &JFactory::getDocument(); // set document for next usage

		foreach ($scripts as $script) {
			if (JFile::exists($ajau_folder . DS . 'js' . DS . $script)) {
				$document->addScript($live_path . 'plugins/system/cdajau/js/' . $script); // default JS to add
			}
		}

		// attach CSS styles
		$document->addStyleSheet($live_path . 'plugins/system/cdajau/css/cdajau.css');
		$document->addStyleSheet($live_path . 'plugins/system/cdajau/js/facebox/facebox.css');
	}
	
	/*
	 * Load extension language
	 */
	function extLanguage($get_option, $ext_folder_path) {
		JPlugin::loadLanguage('plg_system_cdajau_' . $get_option . '', $ext_folder_path);
	}

	/*
	 * Add helper file
	 */
	function addPluginHelper($ext_folder_path)
	{
		$document = &JFactory::getDocument(); // set document for next usage
		$helper = $ext_folder_path . DS . 'helper.php';
		if (JFile::exists($helper)) {
			require_once ($helper);
		}
	}

	/*
	 * Add plugin JS
	 */
	function addPluginScrips($get_option, $ext_folder_path) {
		$live_path = JURI::root(true) . '/'; // define live site
		$script_js = 'jquery.ajau_' . $get_option . '.js'; // name of script

		if (JFile::exists($ext_folder_path . DS . $script_js))
		{
			$document = &JFactory::getDocument(); // set document for next usage
			$document->addScript($live_path . 'plugins/system/cdajau/extensions/' . $get_option . '/' . $script_js); // default JS to add
		}
	}

	/*
	 * Check IE6 function
	 */
	function checkIE6() {
		if (array_key_exists('HTTP_USER_AGENT', $_SERVER)) {
			$is_ie7 = strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'msie 7') !== false;
			$is_ie6 = strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'msie 6') !== false;
			if (!$is_ie7 && $is_ie6) {
				// is internet explorer 6
				return true;
			} else {
				return false;
			}
		}
	}
	
	/*
	 * Filter received elements from buffer - Ajax post
	 */
	function ping($what = 'form') {
		
		// Check for request forgeries
		//JRequest::checkToken() or plgSystemCdAjau::close(JText::_('AJAU_INVALID_TOKEN'));
    	
		$document = &JFactory::getDocument(); // set document for next usage
		$buffer = $document->getBuffer('component');
		
		switch ($what) {
			case 'table':
			default:
				$tag_regex = '#(<table*.?class="adminlist".*?>.*?</table>)#s';
				preg_match($tag_regex, $buffer, $buffer);
		    	$buffer = ( isset($buffer[1]) ? $buffer[1] : '');
				break;
			case 'form':
				// noting to do
				break;
		}
		
    	$mootools_tooltip = "<script type=\"text/javascript\">
            <!--
            var JTooltips = function(){ new Tips($$('.hasTip'), { maxTitleChars: 50, fixed: false}); };
            window.addEvent('domready', JTooltips);
			// -->
			</script>";
    	
    	return $buffer . $mootools_tooltip;
    }
    
    /*
     * Plugin params
     * 
     * Retrieve plugin params related to the plugin
     * 
     * @return string	parameter 
     */
    function pluginParams($paramname = '', $default = '') {
    	// load plugin parameters
		$plugin = &JPluginHelper::getPlugin('system', 'cdajau');
		$plugin = new JParameter($plugin->params);
		
    	if ($paramname) {
    		$value = $plugin->get($paramname, $default);
    		
    		if ($value) {
    			return $value;
    		}
    	}
    	return '';
    }
    
    /*
     * Clear set queues
     * @return void
     */
    
    function clearQueue() {
    	global $mainframe;
    	$mainframe->setUserState('ajau', '');
		$session =& JFactory::getSession();
		$application_queue = $session->get('application.queue', '');
		if ($application_queue) {
			$session->set('application.queue', null);
		}
    }
}
?>